;; Generated package description from expand-region.el  -*- no-byte-compile: t -*-
(define-package "expand-region" "1.0.0.0.20241217.184034" "Increase selected region by semantic units." '((emacs "24.4")) :commit "351279272330cae6cecea941b0033a8dd8bcc4e8" :authors '(("Magnar Sveen" . "magnars@gmail.com")) :maintainer '("Magnar Sveen" . "magnars@gmail.com") :keywords '("marking" "region") :url "https://github.com/magnars/expand-region.el")
